import { CssProperty, InheritedCssProperty, Property, Style, View, Color, ImageAsset, ImageSource, CoreTypes } from '@nativescript/core';
export declare enum Transition {
    Fade = "fade",
    None = "none"
}
export declare enum Priority {
    Low = 0,
    Normal = 1,
    High = 2
}
export declare const progressProperty: Property<ImageCacheItBase, number>;
export declare const loadModeProperty: Property<ImageCacheItBase, "sync" | "async">;
export declare const imageSourceProperty: Property<ImageCacheItBase, ImageSource>;
export declare const srcProperty: Property<ImageCacheItBase, any>;
export declare const placeHolderProperty: Property<ImageCacheItBase, any>;
export declare const errorHolderProperty: Property<ImageCacheItBase, string>;
export declare const stretchProperty: Property<ImageCacheItBase, CoreTypes.ImageStretchType>;
export declare const filterProperty: CssProperty<Style, string>;
export declare const transitionProperty: Property<ImageCacheItBase, Transition>;
export declare const fallbackProperty: Property<ImageCacheItBase, any>;
export declare const priorityProperty: Property<ImageCacheItBase, Priority>;
export declare const tintColorProperty: InheritedCssProperty<Style, string | Color>;
export declare const overlayColorProperty: CssProperty<Style, string | Color>;
export declare const headersProperty: Property<ImageCacheItBase, Map<string, string>>;
export declare const isLoadingProperty: Property<ImageCacheItBase, boolean>;
export declare const decodeWidthProperty: Property<ImageCacheItBase, number>;
export declare const decodeHeightProperty: Property<ImageCacheItBase, number>;
export * from '@nativescript/core/ui/core/view';
export declare class ImageCacheItBase extends View {
    imageSource: ImageSource;
    src: any;
    placeHolder: any;
    errorHolder: any;
    stretch: CoreTypes.ImageStretchType;
    filter: string;
    transition: Transition;
    fallback: any;
    priority: Priority;
    loadMode: 'async' | 'sync';
    isLoading: boolean;
    progress: number;
    overlayColor: string | Color;
    decodeWidth: number;
    decodeHeight: number;
    get tintColor(): Color | string;
    set tintColor(value: Color | string);
    headers: Map<string, string>;
    static onLoadStartEvent: string;
    static onProgressEvent: string;
    static onErrorEvent: string;
    static onLoadEndEvent: string;
    _emitLoadStartEvent(url?: string): void;
    _emitProgressEvent(loaded: number, total: number, progress: number, url: string): void;
    _emitErrorEvent(message: string, url: string): void;
    _emitLoadEndEvent(url?: string, image?: any): void;
    /**
     * @internal
     */
    _createImageSourceFromSrc(value: string | ImageSource | ImageAsset): void;
    private _setImageSource;
}
declare module '@nativescript/core/ui/styling/style' {
    interface Style {
        filter: string;
        overLayColor: string | Color;
    }
}
