var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var ImageCacheItBase_1;
import { booleanConverter, CssProperty, CSSType, InheritedCssProperty, Property, Style, View, Color, ImageAsset, ImageSource, isIOS, Trace, Utils } from '@nativescript/core';
export var Transition;
(function (Transition) {
    Transition["Fade"] = "fade";
    Transition["None"] = "none";
})(Transition || (Transition = {}));
export var Priority;
(function (Priority) {
    Priority[Priority["Low"] = 0] = "Low";
    Priority[Priority["Normal"] = 1] = "Normal";
    Priority[Priority["High"] = 2] = "High";
})(Priority || (Priority = {}));
export const progressProperty = new Property({
    name: 'progress',
    defaultValue: 0
});
export const loadModeProperty = new Property({
    name: 'loadMode',
    defaultValue: 'async',
});
export const imageSourceProperty = new Property({
    name: 'imageSource',
});
export const srcProperty = new Property({
    name: 'src',
});
export const placeHolderProperty = new Property({
    name: 'placeHolder'
});
export const errorHolderProperty = new Property({
    name: 'errorHolder'
});
export const stretchProperty = new Property({
    name: 'stretch',
    affectsLayout: isIOS,
});
export const filterProperty = new CssProperty({
    name: 'filter',
    cssName: 'filter'
});
export const transitionProperty = new Property({
    name: 'transition',
    defaultValue: Transition.None
});
export const fallbackProperty = new Property({
    name: 'fallback',
});
export const priorityProperty = new Property({
    name: 'priority',
    defaultValue: Priority.Normal
});
export const tintColorProperty = new InheritedCssProperty({
    name: 'tintColor',
    cssName: 'tint-color',
    equalityComparer: Color.equals, valueConverter: (value) => new Color(value)
});
export const overlayColorProperty = new CssProperty({
    name: 'overLayColor',
    cssName: 'overlay-color',
    equalityComparer: Color.equals, valueConverter: (value) => new Color(value)
});
export const headersProperty = new Property({
    name: 'headers'
});
export const isLoadingProperty = new Property({
    name: 'isLoading',
    defaultValue: false,
    valueConverter: booleanConverter,
});
export const decodeWidthProperty = new Property({
    name: 'decodeWidth'
});
export const decodeHeightProperty = new Property({
    name: 'decodeHeight',
});
export * from '@nativescript/core/ui/core/view';
let ImageCacheItBase = ImageCacheItBase_1 = class ImageCacheItBase extends View {
    get tintColor() {
        return this.style.tintColor;
    }
    set tintColor(value) {
        if (typeof value === 'string') {
            this.style.tintColor = new Color(value);
        }
        else {
            this.style.tintColor = value;
        }
    }
    _emitLoadStartEvent(url) {
        this.notify({
            eventName: ImageCacheItBase_1.onLoadStartEvent,
            object: this,
            url
        });
    }
    _emitProgressEvent(loaded, total, progress, url) {
        this.notify({
            eventName: ImageCacheItBase_1.onProgressEvent,
            object: this,
            loaded,
            total,
            progress,
            url
        });
    }
    _emitErrorEvent(message, url) {
        this.notify({
            eventName: ImageCacheItBase_1.onErrorEvent,
            object: this,
            message,
            url
        });
    }
    _emitLoadEndEvent(url, image) {
        this.notify({
            eventName: ImageCacheItBase_1.onLoadEndEvent,
            object: this,
            image,
            url
        });
    }
    /**
     * @internal
     */
    _createImageSourceFromSrc(value) {
        const originalValue = value;
        const sync = this.loadMode === 'sync';
        if (typeof value === 'string' || value instanceof String) {
            value = value.trim();
            this.imageSource = null;
            this['_url'] = value;
            this.isLoading = true;
            const imageLoaded = (source) => {
                if (this.src !== originalValue) {
                    return;
                }
                this._setImageSource(source);
            };
            if (Utils.isFontIconURI(value)) {
                const fontIconCode = value.split('//')[1];
                if (fontIconCode !== undefined) {
                    // support sync mode only
                    const font = this.style.fontInternal;
                    const color = this.style.color;
                    imageLoaded(ImageSource.fromFontIconCodeSync(fontIconCode, font, color));
                }
            }
            else if (Utils.isDataURI(value)) {
                const base64Data = value.split(',')[1];
                if (base64Data !== undefined) {
                    if (sync) {
                        imageLoaded(ImageSource.fromBase64Sync(base64Data));
                    }
                    else {
                        ImageSource.fromBase64(base64Data).then(imageLoaded);
                    }
                }
            }
            else if (Utils.isFileOrResourcePath(value)) {
                if (value.indexOf(Utils.RESOURCE_PREFIX) === 0) {
                    const resPath = value.substring(Utils.RESOURCE_PREFIX.length);
                    if (sync) {
                        imageLoaded(ImageSource.fromResourceSync(resPath));
                    }
                    else {
                        this.imageSource = null;
                        ImageSource.fromResource(resPath).then(imageLoaded);
                    }
                }
                else {
                    if (sync) {
                        imageLoaded(ImageSource.fromFileSync(value));
                    }
                    else {
                        this.imageSource = null;
                        ImageSource.fromFile(value).then(imageLoaded);
                    }
                }
            }
            else {
                this.imageSource = null;
                ImageSource.fromUrl(value).then((r) => {
                    if (this['_url'] === value) {
                        this._setImageSource(r, value);
                    }
                }, (err) => {
                    // catch: Response content may not be converted to an Image
                    this.isLoading = false;
                    if (Trace.isEnabled()) {
                        if (typeof err === 'object' && err.message) {
                            err = err.message;
                        }
                        Trace.write(err, Trace.categories.Debug);
                    }
                });
            }
        }
        else if (value instanceof ImageSource) {
            // Support binding the imageSource trough the src property
            this._setImageSource(value);
        }
        else if (value instanceof ImageAsset) {
            ImageSource.fromAsset(value).then((result) => {
                this._setImageSource(result);
            });
        }
        else {
            this._setImageSource(new ImageSource(value));
        }
    }
    _setImageSource(source, url) {
        this.imageSource = source;
        this.isLoading = false;
        this._emitLoadEndEvent(url || this.src);
    }
};
ImageCacheItBase.onLoadStartEvent = 'loadStart';
ImageCacheItBase.onProgressEvent = 'progress';
ImageCacheItBase.onErrorEvent = 'error';
ImageCacheItBase.onLoadEndEvent = 'loadEnd';
ImageCacheItBase = ImageCacheItBase_1 = __decorate([
    CSSType('ImageCacheIt')
], ImageCacheItBase);
export { ImageCacheItBase };
progressProperty.register(ImageCacheItBase);
isLoadingProperty.register(ImageCacheItBase);
loadModeProperty.register(ImageCacheItBase);
imageSourceProperty.register(ImageCacheItBase);
srcProperty.register(ImageCacheItBase);
placeHolderProperty.register(ImageCacheItBase);
errorHolderProperty.register(ImageCacheItBase);
stretchProperty.register(ImageCacheItBase);
filterProperty.register(Style);
transitionProperty.register(ImageCacheItBase);
fallbackProperty.register(ImageCacheItBase);
priorityProperty.register(ImageCacheItBase);
tintColorProperty.register(Style);
headersProperty.register(ImageCacheItBase);
overlayColorProperty.register(Style);
decodeWidthProperty.register(ImageCacheItBase);
decodeHeightProperty.register(ImageCacheItBase);
//# sourceMappingURL=common.js.map